package com.sams.jws.chapter15;
/**
 * DocumentUtils
 * Utility methods for working with JAXM SOAP documents
 */
import java.util.Iterator;
import javax.xml.soap.SOAPElement;

public class DocumentUtils {
/**
 * Private constructor ensures that util class can't be instantiated or extended.
 */
private DocumentUtils() {}
/**
 * findChild
 * Return the SOAPElement representing direct decendant of root 
 * named by childName. This is case-sensitive.
 */
public static SOAPElement findChild(SOAPElement root, String childName) {
    Iterator i = root.getChildElements();
    while (i.hasNext()) {
        SOAPElement soapElement = (SOAPElement) i.next();
        if (soapElement.getElementName().getLocalName().equals(childName))
            return soapElement;
    } 
    return null;
}	// findChild()
/**
 * findValueOfChild
 * Find the SOAPElement representing direct decendant of root 
 * named by childName. If the immediate decendant of this is a text node, 
 * return the value of this node. Otherwise return null.
 *
 * For example, given: <Foo> bar </Foo> where Foo is the root, return bar.
 * Note that childName is case sensitive
 */
public static String findValueOfChild(SOAPElement root, String childName) {
    SOAPElement childElement = findChild(root, childName);
    if (childElement == null) {
        return null;
    }
    return childElement.getValue();
}	// findValueOfChild()
}	// DocumentUtils
