package com.sams.jws.chapter15;
/*
 * HelloWorldSender
 * Very basic demonstration of JAXM SOAP profile with no provider. 
 * This shows synchronous, client/server-style call.
 */
// General JDK
import java.io.IOException;
// JAXM
import javax.xml.soap.SOAPConnectionFactory;
import javax.xml.soap.SOAPConnection;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPPart;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPHeaderElement;
import javax.xml.soap.Name;
import javax.xml.soap.SOAPException;
import javax.xml.messaging.URLEndpoint;

public class HelloWorldSender {

    public static void main(String args[]) {

        URLEndpoint urlEndpoint = null;
        if (args.length == 1)
            urlEndpoint = new URLEndpoint(args[0]);
        else {
            System.err.println("Usage: java HelloWorldSender targetURL");
            System.exit(0);
        } // Else

        try {
            // Initialize connection and SOAP Profile.
            System.out.println("Opening connection...");
            SOAPConnectionFactory soapConnFct = SOAPConnectionFactory.newInstance();
            SOAPConnection soapConnection = soapConnFct.createConnection();

            System.out.println("Creating message...");
            MessageFactory messageFactory = MessageFactory.newInstance();
            SOAPMessage requestSoapMsg = messageFactory.createMessage();

            // Create references to SOAP message components 
            SOAPPart requestSoapPart = requestSoapMsg.getSOAPPart();
            SOAPEnvelope requestSoapEnv = requestSoapPart.getEnvelope();
            SOAPHeader requestSoapHdr = requestSoapEnv.getHeader();

            // Add header field. Header fields are optional.
            Name questionName = requestSoapEnv.createName("Question");
            SOAPHeaderElement questionElement =
                requestSoapHdr.addHeaderElement(questionName);
            questionElement.setMustUnderstand(true);
            questionElement.addTextNode("Are you there?");

            /*
            * Note that a body element is not defined here. 
            * Because this is a required field, JAXM 
            * implicitly creates an empty body.
            */

            // Save all changes to message before printing. 
            // Sending will also implictly save changes.
            requestSoapMsg.saveChanges();

            System.out.println("HelloWorldSender is sending the following message:");
            try {
                requestSoapMsg.writeTo(System.out);
            } catch (IOException e) {
                System.err.println(
                    "Encountered IOException trying to write SOAPMessage to System.out: ");
                e.printStackTrace();
            }
            System.out.println("\nTo URL: ");
            System.out.println(urlEndpoint.getURL() + "\n");

            // Send request and block until reply appears.
            SOAPMessage responseSoapMsg = soapConnection.call(requestSoapMsg, urlEndpoint);

            System.out.println("HelloWorldSender received the following reponse message:");
            try {
                responseSoapMsg.writeTo(System.out);
                System.out.println("\n");
            } catch (IOException e) {
                System.err.println(
                    "Encountered IOException trying to write SOAPMessage to System.out: ");
                e.printStackTrace();
            }

            // Clean up
            soapConnection.close();
            System.out.println("Connection closed.");

        } catch (SOAPException e) {
            System.err.println("Encountered general SOAPException: ");
            e.printStackTrace();
        }
        System.out.println("Done.");
    } // main()
} // HelloWorldSender