import java.security.*;
import java.security.interfaces.*;
import java.io.*;

/** Computes a digital signature for a file */

public class SignData
{
    public static void main(String[] args)
    {
        try
        {
            String keystorePassword = "mykspass";
            String testAlias = "signkey";
            String testKeyPassword = "mykeypass";

// Create a keystore.
            KeyStore keystore = KeyStore.getInstance("JKS");

// Figure out where the user's keystore is located.
            String keystoreFilename = System.getProperty("user.home")+
                File.separator+".keystore";
                
// Load the keystore from the keystore file.
            keystore.load(new FileInputStream(keystoreFilename),
                keystorePassword.toCharArray());

// Locate the key with the specified alias and password.
            Key testkey = keystore.getKey(testAlias,
                testKeyPassword.toCharArray());

// Assume that the key is an RSA key.
            RSAPrivateKey pvtKey = (RSAPrivateKey) testkey;

// Get the certificate for the key (not for computing the signature,
// just for writing out to a separate file for later signature verification).
            java.security.cert.Certificate cert =
                keystore.getCertificate(testAlias);

// Create and initialize a signer to use MD5 and RSA.
            Signature signer = Signature.getInstance("MD5withRSA");
            signer.initSign(pvtKey);

// Open the data file.
            FileInputStream in = new FileInputStream(args[0]);

// Create a block of bytes for reading the file.
            byte[] buffer = new byte[4096];
            int len;

// Read a block of the file and add it to the signature object.
            while ((len = in.read(buffer)) > 0)
            {
                signer.update(buffer, 0, len);
            }
            in.close();

// Compute the digital signature.
            byte signatureBytes[] = signer.sign();

// Write the signature out to a file.
            FileOutputStream out = new FileOutputStream(args[0]+".sig");
            out.write(signatureBytes);
            out.close();

// Write the certificate out to the file in encoded format.
            out = new FileOutputStream(args[0]+".cer");
            out.write(cert.getEncoded());
            out.close();
        }
        catch (Exception exc)
        {
            exc.printStackTrace();
        }
    }
}
